﻿/**
 * Copyright 1998-2008 Epic Games, Inc. All Rights Reserved.
 *
 * Tab page for a set of options to filter join game server results by.
 */

class UTUITabPage_ServerFilter extends UTUITabPage_Options
	placeable;

`include(Core/Globals.uci)

/** Reference to the menu items datastore. */
var UTUIDataStore_MenuItems	MenuDataStore;

/** Reference to the game search datastore. */
var UTDataStore_GameSearchDM SearchDataStore;

/** Indicates that the list of options are out of date and need to be regenerated */
var	private transient bool bOptionsDirty;

/** Called when the user changes the game type */
delegate transient OnSwitchedGameType();

/* == Events == */
/** Post initialization event - Setup widget delegates.*/
event PostInitialize()
{
	Super.PostInitialize();

	// Set the button tab caption.
	SetDataStoreBinding("<Strings:UTGameUI.JoinGame.Filter>");

	MenuDataStore = UTUIDataStore_MenuItems(UTUIScene(GetScene()).FindDataStore('UTMenuItems'));
	SearchDataStore = UTDataStore_GameSearchDM(UTUIScene(GetScene()).FindDataStore('UTGameSearch'));
}

/**
 * Causes this page to become (or no longer be) the tab control's currently active page.
 *
 * @param	PlayerIndex	the index [into the Engine.GamePlayers array] for the player that wishes to activate this page.
 * @param	bActivate	TRUE if this page should become the tab control's active page; FALSE if it is losing the active status.
 * @param	bTakeFocus	specify TRUE to give this panel focus once it's active (only relevant if bActivate = true)
 *
 * @return	TRUE if this page successfully changed its active state; FALSE otherwise.
 */
event bool ActivatePage( int PlayerIndex, bool bActivate, optional bool bTakeFocus=true )
{
	local bool bResult;
	local UIObject SelectedOption;

	bResult = Super.ActivatePage(PlayerIndex, bActivate, bTakeFocus);

	if ( bResult && bActivate )
	{
		if ( bOptionsDirty )
		{
			OptionList.RefreshAllOptions();
			bOptionsDirty = false;
		}

		SelectedOption = OptionList.GetCurrentlySelectedOption();

		// if the currently selected option is disabled, select the next item
		if ( !SelectedOption.IsEnabled(PlayerIndex) )
		{
			OptionList.SelectNextItem(true);
		}

		ShowHideGameMode(PlayerIndex);
	}

	return bResult;
}

/**
 * Marks the options list as out of date and if visible, refreshes the options list.  Called when the user
 * changes the gametype.
 */
function MarkOptionsDirty()
{
	if ( IsVisible() )
	{
		OptionList.RefreshAllOptions();
		bOptionsDirty = false;
	}
	else
	{
		bOptionsDirty = true;
	}
}

/**
 * Enables / disables the "match type" control based on whether we are signed in online.
 */
function ValidateServerType()
{
	local int PlayerIndex, PlayerControllerID;
	local UTUIScene UTOwnerScene;

	UTOwnerScene = UTUIScene(GetScene());
	if ( UTOwnerScene != None && StringListDataStore != None )
	{
		// find the "MatchType" control (contains the "LAN" and "Internet" options);  if we aren't signed in online,
		// don't have a link connection, or not allowed to play online, don't allow them to select one.
		PlayerIndex = UTOwnerScene.GetPlayerIndex();
		PlayerControllerID = UTOwnerScene.GetPlayerControllerId( PlayerIndex );
		if ( !IsLoggedIn(PlayerControllerID,true) || !UTOwnerScene.CheckOnlinePrivilegeAndError(PlayerControllerID) )
		{
			ForceLANOption(PlayerIndex);
		}
	}
}

final function ForceLANOption( int PlayerIndex )
{
	local UIObject ServerTypeOption, GameTypeOption, AllGameTypesOption;
	local int ValueIndex;
	local name MatchTypeName;

	ServerTypeOption = FindChild('MatchType', true);
	if ( ServerTypeOption != None )
	{
		MatchTypeName = IsConsole(CONSOLE_XBox360) ? 'MatchType360' : 'MatchType';
		ValueIndex = StringListDataStore.GetCurrentValueIndex(MatchTypeName);
		if ( ValueIndex != class'UTUITabPage_ServerBrowser'.const.SERVERBROWSER_SERVERTYPE_LAN )
		{
			// make sure the "LAN" option is selected
			StringListDataStore.SetCurrentValueIndex(MatchTypeName,class'UTUITabPage_ServerBrowser'.const.SERVERBROWSER_SERVERTYPE_LAN);
			UIDataStoreSubscriber(ServerTypeOption).RefreshSubscriberValue();
		}

		// LAN queries show all gametypes, so switch to "DeathMatch" so that search results are always in the same place
		ValueIndex = MenuDataStore.FindValueInProviderSet('GameModeFilter', 'GameSearchClass', "UTGameSearchDM");
		if ( ValueIndex != INDEX_NONE && MenuDataStore.GameModeFilter != ValueIndex )
		{
			MenuDataStore.GameModeFilter = ValueIndex;
			MarkOptionsDirty();
			SearchDataStore.SetCurrentByName('UTGameSearchDM', true);
		}

		GameTypeOption = FindChild('GameModeFilter', true);
		AllGameTypesOption = FindChild('ListAllGameModes', true);

		// use the accessor so that if the match or server type options are selected, we select the next possible one
		OptionList.EnableItem(PlayerIndex, ServerTypeOption, false);
		OptionList.EnableItem(PlayerIndex, GameTypeOption, false);
		OptionList.EnableItem(PlayerIndex, AllGameTypesOption, false);
	}
}

/** Update the gamemode visibility depending on LAN/Internet setting */
function ShowHideGameMode(int PlayerIndex)
{
	local int ValueIndex;
	local UIObject GameModeOption, FullServerOption, AllGameModesOption;
	local bool bLANMode, bAllGameModes;
	local string OptStr;

	GameModeOption = FindChild('GameModeFilter',true);

	if ( GameModeOption != None )
	{
		// Determine whether LAN has been selected
		ValueIndex = StringListDataStore.GetCurrentValueIndex(IsConsole(CONSOLE_XBox360) ? 'MatchType360' : 'MatchType');
		bLANMode = ValueIndex == class'UTUITabPage_ServerBrowser'.const.SERVERBROWSER_SERVERTYPE_LAN;

		// Determine whethr 'List All Game Modes' has been selected
		if (!bLANMode && GetDataStoreStringValue("<Registry:ListAllGameModes>", OptStr))
			bAllGameModes = bool(OptStr);


		// if the user wants to search for LAN matches, disable the gametype combo
		OptionList.EnableItem(PlayerIndex, GameModeOption, !(bLANMode || bAllGameModes));


		// Disable 'List All Game Modes' if LAN mode is enabled
		AllGameModesOption = FindChild('ListAllGameModes', true);

		if (AllGameModesOption != none)
			OptionList.EnableItem(PlayerIndex, AllGameModesOption, !bLANMode);


		FullServerOption = FindChild('ShowFullServers',true);
		if ( FullServerOption != None )
		{
			// if the user wants to search for LAN matches, disable the fullservers option (it is not supported)
			OptionList.EnableItem(PlayerIndex, FullServerOption, !bLANMode);
		}
	}
}

/** Pass through the option callback. */
function OnOptionList_OptionChanged(UIScreenObject InObject, name OptionName, int PlayerIndex)
{
	local string OutStringValue, GameClassValue;
	local int ProviderIdx;
	local array<UIDataStore> OutDataStores;
	local UIDataStorePublisher Publisher;
	local string OptStr;

	Super.OnOptionList_OptionChanged(InObject, OptionName, PlayerIndex);

	// if we're the active page, publish this option's value to its data store so that the value won't be reset
	// if the option list is regenerated
	if ( IsActivePage() || GetOwnerTabControl() == None )
	{
		Publisher = UIDataStorePublisher(InObject);
		if ( Publisher != None )
		{
			Publisher.SaveSubscriberValue(OutDataStores);
		}
	}

	if (OptionName == 'AllGameModes_Client')
	{
		if (UICheckBox(InObject).IsChecked())
		{
			SearchDataStore.CustomGameTypeClass = "";
			SearchDataStore.SetCurrentByName('UTGameSearchCustom', false);
		}
		else
		{
			// If 'List All Game Modes' was unchecked, then refresh the search data store
			OptionName = 'GameMode_Client';
		}

		ShowHideGameMode(PlayerIndex);
	}

	if(OptionName=='GameMode_Client')
	{
		GetDataStoreStringValue("<Registry:ListAllGameModes>", OptStr);

		if (!bool(OptStr) && GetDataStoreStringValue("<UTMenuItems:GameModeFilterClass>", OutStringValue))
		{
			// make sure to update the GameSettings value - this is used to build the join URL
			SetDataStoreStringValue("<UTGameSettings:CustomGameMode>", OutStringValue);
			GameClassValue = OutStringValue;

			// find the index into the UTMenuItems data store for the gametype with the specified class name
			ProviderIdx = MenuDataStore.FindValueInProviderSet('GameModeFilter','GameMode', OutStringValue);

			// now that we know the index into the UTMenuItems data store, we can retrieve the tag that is used to identify the corresponding
			// game search object in the Game Search data store.
			if(ProviderIdx != INDEX_NONE && MenuDataStore.GetValueFromProviderSet('GameModeFilter','GameSearchClass', ProviderIdx, OutStringValue))
			{
				// If no game search object has been specified, then default to the custom game search object
				if (OutStringValue == "")
					OutStringValue = "UTGameSearchCustom";

				// If the custom game search object has been specified, then supply the search datastore with the game class, for filtering
				if (OutStringValue ~= "UTGameSearchCustom")
					SearchDataStore.CustomGameTypeClass = GameClassValue;

				// Set the search settings class
				SearchDataStore.SetCurrentByName(name(OutStringValue), false);
			}

			// fire the delegate
			OnSwitchedGameType();

			MarkOptionsDirty();
		}
	}
	else if ( OptionName == 'MatchType' || OptionName == 'MatchType360')
	{
		ShowHideGameMode(PlayerIndex);
	}
}
